library(fda)
library(MASS)
library(Matrix)

### This Function is created by Ruzong Fan, Dec 18, 2012 ###

Trace.Matrix = function(x)
   {
   sum(diag(x))
   }

MetaMFLM_fpca_no_position <- function(L, is.homo = TRUE, pheno, mode = "Additive", geno, covariate, kz = 30, kb = 30, smooth.cov = FALSE) 
   {
   require(mgcv)
   
   #### 
   genoX = idx = dqr = index = nsample = UJ = dimcov = n = p = list(length = L)   
   
   ### Define design matrix W using geneotype for each study 
   for (k in 1:L)
      {
      geno[[k]] [is.na(geno[[k]])] = 0
      covariate[[k]][is.na(covariate[[k]])] = 0
 
      ### define genotyping matrix for Dom and Rec modes ###
      ### For Dom mode, redefine geno[[k]][i,j] = 1 if geno[[k]][i,j] = 1 or 2
      ### For Rec mode, redefine geno[[k]][i,j] = 1 if geno[[k]][i,j] = 2 
      genoX[[k]] = geno[[k]] * 0
      for (i in 1:nrow(geno[[k]]))
          for (j in 1:ncol(geno[[k]]))
             {
             if (mode == "Dom")
                {
                if (geno[[k]][i, j] == 1 || geno[[k]][i, j] == 2)
                   genoX[[k]][i,j] = 1
                }
                else if ( mode == "Rec")
                   if (geno[[k]][i, j] == 2)
                      genoX[[k]][i, j] = 1
             }
         
      if  ( mode == "Rec" || mode == "Dom")
         geno[[k]] = genoX[[k]]     
      
      #idx[[k]]       = is.na(pheno[[k]])
      #pheno[[k]]     = pheno[[k]][!idx[[k]]]
      #geno[[k]]      = geno[[k]][!idx[[k]],]
      #covariate[[k]] = covariate[[k]][!idx[[k]],]
     
      if (is.vector(covariate[[k]]) == FALSE ) 
      {
      #covariate[[k]] = covariate[[k]][!idx[[k]], ]
      dimcov[[k]] = dim(covariate[[k]])[2]
      }else if (is.vector(covariate)) 
         {
         #covariate[[k]] = covariate[[k]][ !idx[[k]] ]
         dimcov[[k]] = 1
         }
             
      nsample[[k]] = nrow(geno[[k]])

      dqr[[k]]     = qr(geno[[k]])
      index[[k]]   = dqr[[k]]$pivot[1:dqr[[k]]$rank]
      geno[[k]]    = geno[[k]][, index[[k]]]

      kb = min(kz, kb)
      n[[k]]  = length(pheno[[k]])
      p[[k]]  = ifelse(is.null(covariate[[k]]), 0, dimcov[[k]])
	
      if (is.matrix(geno[[k]])){
         Funcs = list(length=1)
         Funcs[[1]] = geno[[k]]
         } else {
            Funcs = geno[[k]]
            }
	
      # functional predictors
      N.Pred = length(Funcs)
	
      t = phi = psi = list(length=N.Pred)
      for (i in 1:N.Pred){
         t[[i]] = seq(0, 1, length = dim(Funcs[[i]])[2])
         N_obs  = length(t[[i]])
		
         # de-mean the functions
         meanFunc   = apply(Funcs[[i]], 2, mean, na.rm=TRUE)
         resd       = sapply(1:length(t[[i]]), function(u) Funcs[[i]][,u]-meanFunc[u])
         Funcs[[i]] = resd

         # construct and smooth covariance matrices
         G.sum   <- matrix(0, N_obs, N_obs)
         G.count <- matrix(0, N_obs, N_obs)

         for (j in 1:dim(resd)[1]){
            row.ind = j
    	      temp    = resd[row.ind, ] %*% t( resd[row.ind, ])
            G.sum   <- G.sum + replace(temp, which(is.na(temp)), 0)
            G.count <- G.count + as.numeric(!is.na(temp))
            }
         G <- ifelse(G.count==0, NA,  G.sum/G.count)   
		
         ## get the eigen decomposition of the smoothed variance matrix
         if (smooth.cov){
            G2 <- G
            M  <- length(t[[i]])
            diag(G2)= rep(NA, M)
            g2 <- as.vector(G2)
            ## define a N*N knots for bivariate smoothing
            N <- 10

            ## bivariate smoothing using the gamm function
            t1 <- rep(t[[i]], each=M)
            t2 <- rep(t[[i]], M)
            newdata <- data.frame(t1 = t1, t2 = t2)
            K.0 <- matrix(predict(gam(as.vector(g2) ~ te(t1, t2, k = N)), newdata), M, M) # smooth K.0
            K.0 <- (K.0 + t(K.0)) / 2    
		
            eigenDecomp <- eigen(K.0)
            } else {
               eigenDecomp <- eigen(G)
               }
                
         ### make sure no "Error in eigenDecomp$vectors[, 1:kz] : subscript out of bounds"
         kz1 = min(kz, length(eigenDecomp$values))	
         psi[[i]] = eigenDecomp$vectors[,1:kz1]

         # set the basis to be used for beta(t)
         num = kb-2
         qtiles   <- seq(0, 1, length = num + 2)[-c(1, num + 2)]
         knots    <- quantile(t[[i]], qtiles)
         phi[[i]] = cbind(1, t[[i]], sapply(knots, function(k) ((t[[i]] - k > 0) * (t[[i]] - k))))

         U = matrix(0, nrow=dim(resd)[1], ncol=kz1)

         for (j in 1:dim(resd)[1]){
            U[j,] <-  replace(resd[j,], which(is.na(resd[j,])), 0) %*% psi[[i]][ ,1:kz1 ] 
            }

         J  = t(psi[[i]]) %*% phi[[i]]
         }
      UJ[[k]] = U %*% J
      }

   ### Define over all design matrix  of genetic effect, when the geentic effect is homozygous
   ### Define overall trait values   
   for (k in 1:L)
      {
      if (k == 1)
         { 
         if (is.homo == TRUE)
            {
            U = UJ[[1]]
            } else {}
         Y = pheno[[1]]
         }
         else
            {
            if (is.homo == TRUE)
               {
               U = rbind (U, UJ[[k]])
               } else{}
            Y = rbind(Y, pheno[[k]])
            }
      }
   
   ### Define overall design matrix of genetic effect, when the geentic effect is heterozygous     
   tcov    = dimcov[[1]]
   tsample = nsample[[1]]
   for (k in 2:L)
      {
      tcov    = tcov + dimcov[[k]]
      tsample = tsample + nsample[[k]]
      }

   if (is.homo == FALSE)
      {
      tmp = L * kb
      U = matrix (0, nrow = tsample, ncol = tmp)  
      trow = 1
      for (k in 1:L)
         {
         tcol = kb * (k-1) + 1      
         for (i in trow:(trow + nsample[[k]] - 1))
            for (j in tcol:(tcol + kb - 1))
               {                 
               U[i, j] = UJ[[k]][i - trow + 1, j - tcol + 1]
               } 
         trow = trow + nsample[[k]]
         }
      }   

   ### Define covariate matrix, the covariate effects are assumed to be different from study to study
   cov = matrix (0, nrow = tsample, ncol = tcov)
   trow = tcol = 1
   for (k in 1:L)
      {
      for (i in trow:(trow + nsample[[k]] - 1))
          for (j in tcol:(tcol + dimcov[[k]] - 1))
             {
             if (is.vector(covariate[[k]]) == FALSE )
                {
                cov[i, j] = covariate[[k]][i - trow + 1, j - tcol + 1]
                }
                if (is.vector(covariate[[k]]) == TRUE )
                {
                cov[i, j] = covariate[[k]][i - trow + 1]
                }
                else {}
             }
      if (is.vector(covariate[[k]]) == FALSE )
         {
         trow = trow + nsample[[k]]
         tcol = tcol + dimcov[[k]] 
         } 
         if (is.vector(covariate[[k]]) == TRUE )
                {
                trow = trow + nsample[[k]]
                tcol = tcol + 1 
                }
                else {}
      }

   #########
   pval = list()
       	
   fit = lm (Y ~ cov + U)
   if ( dim(Y)[2] > 1){
      pval$Pillai           = anova(fit, test = "Pillai")[3,6]
      pval$Wilks            = anova(fit, test = "Wilks")[3,6]
      pval$Hotelling_Lawley = anova(fit, test = "Hotelling-Lawley")[3,6]
      pval$Roy              = anova(fit, test = "Roy")[3,6]
      pval$Spherical        = anova(fit, test = "Spherical")[3,5]
      pval$Spherical_GG     = anova(fit, test = "Spherical")[3,6]
      pval$Spherical_HF     = anova(fit, test = "Spherical")[3,7]
      }else if ( dim(Y)[2] == 1){ ### In this case, the following results are the same ###
         pval$F                = anova(fit, test = "F")[2,5]
         pval$Pillai           = anova(fit, test = "Pillai")[2,5]
         pval$Wilks            = anova(fit, test = "Wilks")[2,5]
         pval$Hotelling_Lawley = anova(fit, test = "Hotelling-Lawley")[2,5]
         pval$Roy              = anova(fit, test = "Roy")[2,5]
         pval$Spherical        = anova(fit, test = "Spherical")[2,5]
         } 

   pval     	
   }

