library(fda)
library(MASS)
library(Matrix)

### This Function is modified by Ruzong Fan, March 20, 2013 ###
MetaMFLM_beta_smooth_only=function(L, is.homo = TRUE, pheno, mode = "Additive", geno, pos, order, basis, covariate, base = "bspline", interaction = FALSE)  
   {
   if (base ==  "bspline"){
      betabasis  = create.bspline.basis(norder = order, nbasis = basis)
      } else if (base == "fspline"){
      betabasis  = create.fourier.basis(c(0,1), nbasis = basis)
      }else { }

   genoX = idx = dqr = index = nsample = dimcov = UJ = list(length = L)   
   
   ### Define design matrix W using geneotype for each study 
   for (k in 1:L)
      {
      geno[[k]] [is.na(geno[[k]])] = 0
      covariate[[k]][is.na(covariate[[k]])] = 0
 
      ### define genotyping matrix for Dom and Rec modes ###
      ### For Dom mode, redefine geno[[k]][i,j] = 1 if geno[[k]][i,j] = 1 or 2
      ### For Rec mode, redefine geno[[k]][i,j] = 1 if geno[[k]][i,j] = 2 
      genoX[[k]] = geno[[k]] * 0
      for (i in 1:nrow(geno[[k]]))
          for (j in 1:ncol(geno[[k]]))
             {
             if (mode == "Dom")
                {
                if (geno[[k]][i, j] == 1 || geno[[k]][i, j] == 2)
                   genoX[[k]][i,j] = 1
                }
                else if ( mode == "Rec")
                   if (geno[[k]][i, j] == 2)
                      genoX[[k]][i, j] = 1
             }
         
      if  ( mode == "Rec" || mode == "Dom")
         geno[[k]] = genoX[[k]]     
      
      #idx[[k]]       = is.na(pheno[[k]])
      #pheno[[k]]     = pheno[[k]][!idx[[k]]]
      #geno[[k]]      = geno[[k]][!idx[[k]],]
      #covariate[[k]] = covariate[[k]][!idx[[k]],]
   
      dqr[[k]]     = qr(geno[[k]])
      index[[k]]   = dqr[[k]]$pivot[1:dqr[[k]]$rank]
      geno[[k]]    = geno[[k]][, index[[k]]]
      pos[[k]]     = pos[[k]][index[[k]]]
      if (is.vector(covariate[[k]]) == FALSE )
         {
         dimcov[[k]]    = ncol(covariate[[k]])
         }
         else if (is.vector(covariate[[k]]) == TRUE )
            {
            dimcov[[k]] = 1
            }
            else 
               {
               dimcov[[k]] = 0
               }

      nsample[[k]] = nrow(geno[[k]])

      if (max(pos[[k]]) > 1) {
         pos[[k]] = (pos[[k]] - min(pos[[k]])) / (max(pos[[k]]) - min(pos[[k]]))
	       }
 	    
      B = eval.basis(pos[[k]], betabasis)
	
      UJ[[k]] = geno[[k]] %*% B
      }

   ### Define over all design matrix  of genetic effect, when the geentic effect is homozygous
   ### Define overall trait values   
   for (k in 1:L)
      {
      if (k == 1)
         { 
         if (is.homo == TRUE)
            {
            U = UJ[[1]]
            } else {}
         Y = pheno[[1]]
         }
         else
            {
            if (is.homo == TRUE)
               {
               U = rbind (U, UJ[[k]])
               } else{}
            Y = rbind(Y, pheno[[k]])
            }
      }
   
   ### Define overall design matrix of genetic effect, when the geentic effect is heterozygous     
   tcov    = dimcov[[1]]
   tsample = nsample[[1]]
   for (k in 2:L)
      {
      tcov    = tcov + dimcov[[k]]
      tsample = tsample + nsample[[k]]
      }

   if (is.homo == FALSE)
      {
      tmp = L * basis
      U = matrix (0, nrow = tsample, ncol = tmp)  
      trow = 1
      for (k in 1:L)
         {
         tcol = basis * (k-1) + 1      
         for (i in trow:(trow + nsample[[k]] - 1))
            for (j in tcol:(tcol + basis - 1))
               {                 
               U[i, j] = UJ[[k]][i - trow + 1, j - tcol + 1]
               } 
         trow = trow + nsample[[k]]
         }
      }   

   ### Define covariate matrix, the covariate effects are assumed to be different from study to study
   cov = matrix (0, nrow = tsample, ncol = tcov)
   trow = tcol = 1
   for (k in 1:L)
      {
      for (i in trow:(trow + nsample[[k]] - 1))
          for (j in tcol:(tcol + dimcov[[k]] - 1))
             {
             if (is.vector(covariate[[k]]) == FALSE )
                {
                cov[i, j] = covariate[[k]][i - trow + 1, j - tcol + 1]
                }
                if (is.vector(covariate[[k]]) == TRUE )
                {
                cov[i, j] = covariate[[k]][i - trow + 1]
                }
                else {}
             }
      if (is.vector(covariate[[k]]) == FALSE )
         {
         trow = trow + nsample[[k]]
         tcol = tcol + dimcov[[k]] 
         } 
         if (is.vector(covariate[[k]]) == TRUE )
                {
                trow = trow + nsample[[k]]
                tcol = tcol + 1 
                }
                else {}
      }

   #########
   pval = list()
   
   if (interaction == FALSE) 
       {        
       fit        = lm (Y ~ cov + U)
       if ( dim(Y)[2] > 1){
          pval$Pillai           = anova(fit, test = "Pillai")[3,6]
          pval$Wilks            = anova(fit, test = "Wilks")[3,6]
          pval$Hotelling_Lawley = anova(fit, test = "Hotelling-Lawley")[3,6]
          pval$Roy              = anova(fit, test = "Roy")[3,6]
          pval$Spherical        = anova(fit, test = "Spherical")[3,5]
          pval$Spherical_GG     = anova(fit, test = "Spherical")[3,6]
          pval$Spherical_HF     = anova(fit, test = "Spherical")[3,7]
          }
          else if ( dim(Y)[2] == 1){ ### In this case, the following results are the same ###
             pval$F                = anova(fit, test = "F")[2,5]
             pval$Pillai           = anova(fit, test = "Pillai")[2,5]
             pval$Wilks            = anova(fit, test = "Wilks")[2,5]
             pval$Hotelling_Lawley = anova(fit, test = "Hotelling-Lawley")[2,5]
             pval$Roy              = anova(fit, test = "Roy")[2,5]
             pval$Spherical        = anova(fit, test = "Spherical")[2,5]
             } 
       }else ### The following codes are for dim(pheno)[2] > 1 only ####
          {
          Interaction = matrix (0, ncol =  ncol(U), nrow =nrow(U) )
          for ( i in 1:nrow(Interaction) ){ Interaction[i,] = covariate[i,2] * U[i,] }
          
          fitInt     = lm (Y ~ cov + U + Interaction)
          pval$Pillai           = anova(fitInt, test = "Pillai")[4,6]
          pval$Wilks            = anova(fitInt, test = "Wilks")[4,6]
          pval$Hotelling_Lawley = anova(fitInt, test = "Hotelling-Lawley")[4,6]
          pval$Roy              = anova(fitInt, test = "Roy")[4,6]
          pval$Spherical        = anova(fitInt, test = "Spherical")[4,5]                   
          pval$Spherical_GG     = anova(fitInt, test = "Spherical")[4,6]
          pval$Spherical_HF     = anova(fitInt, test = "Spherical")[4,7]             
          } 
   pval     	
   }

