library(fda)
library(MASS)
library(Matrix)
library(MetaSKAT)

source("C:/NICHD/Research/software/Fan/Func_Linear_Model/MFLM_meta/MetaMFLM_beta_smooth_only.R")
source("C:/NICHD/Research/software/Fan/Func_Linear_Model/MFLM_meta/MetaMFLM_fixed_model.R")
source("C:/NICHD/Research/software/Fan/Func_Linear_Model/MFLM_meta/MetaMFLM_FPCA.R")
source("C:/NICHD/Research/software/Fan/Func_Linear_Model/MFLM_meta/MetaMFLM_FPCA_no_position.R")
source("C:/NICHD/Research/software/Fan/Func_Linear_Model/MFLM_meta/MetaMFLM_additive_effect_model.R")
source("C:/NICHD/Research/software/Fan/Func_Linear_Model/MFLM_meta/MetaSKAT_Example_cov_functions.R")

data(Example)
names(Example)

attach(Example)

##########################
gbasis =  25
bbasis =  15
fbasis =  25
order  =   4

L = 3
kz = 20
kb = 10

#######################################################
### Define var-covariance matrix of the error terms ###  
#######################################################
A = matrix(c(1,0.6,-0.35,0.6,1,-0.45,-0.35,-0.45,1),3,3,byrow=T)

pheno = covariate = geno = pos = list(length = L)
            
covariate[[1]] = Generate_Covariates_1 ( length(y.list[[1]]) )
covariate[[2]] = Generate_Covariates_2 ( length(y.list[[2]]) )
covariate[[3]] = Generate_Covariates_3 ( length(y.list[[3]]) )
      
X1 = Get_Alpha0_Q1(covariate[[1]])
X2 = Get_Alpha0_Q2(covariate[[2]])
X3 = Get_Alpha0_Q3(covariate[[3]]) 
     
pheno[[1]] = X1 + mvrnorm(n = length(y.list[[1]]), mu=rep(0,3), Sigma = A)
pheno[[2]] = X2 + mvrnorm(n = length(y.list[[2]]), mu=rep(0,3), Sigma = A) 
pheno[[3]] = X3 + mvrnorm(n = length(y.list[[3]]), mu=rep(0,3), Sigma = A)

geno[[1]] = Z.list[[1]][1:length(y.list[[1]]), ]
geno[[2]] = Z.list[[2]][1:length(y.list[[2]]), ]
geno[[3]] = Z.list[[3]][1:length(y.list[[3]]), ]

pos[[1]] = c( 1:dim(geno[[1]])[2] )
pos[[2]] = c( 1:dim(geno[[2]])[2] )
pos[[3]] = c( 1:dim(geno[[3]])[2])

smooth.cov = FALSE

###
MetaMFLM_beta_smooth_only(L, is.homo = TRUE, pheno, mode = "Additive", geno, pos, order, bbasis, covariate, base = "bspline", interaction = FALSE)
MetaMFLM_beta_smooth_only(L, is.homo = TRUE, pheno, mode = "Additive", geno, pos, order, fbasis, covariate, base = "fspline", interaction = FALSE)

MetaMFLM_beta_smooth_only(L, is.homo = FALSE, pheno, mode = "Additive", geno, pos, order, bbasis, covariate, base = "bspline", interaction = FALSE)
MetaMFLM_beta_smooth_only(L, is.homo = FALSE, pheno, mode = "Additive", geno, pos, order, fbasis, covariate, base = "fspline", interaction = FALSE)

################
MetaMFLM_fixed_model(L, is.homo = TRUE, pheno, mode = "Additive", geno, pos, order, bbasis, gbasis, covariate, base = "bspline", interaction = FALSE)
MetaMFLM_fixed_model(L, is.homo = TRUE, pheno, mode = "Additive", geno, pos, order, fbasis, gbasis, covariate, base = "fspline", interaction = FALSE)

MetaMFLM_fixed_model(L, is.homo = FALSE, pheno, mode = "Additive", geno, pos, order, bbasis, gbasis, covariate, base = "bspline", interaction = FALSE)
MetaMFLM_fixed_model(L, is.homo = FALSE, pheno, mode = "Additive", geno, pos, order, fbasis, gbasis, covariate, base = "fspline", interaction = FALSE)

###
MetaMFLM_fpca_no_position(L, is.homo = TRUE, pheno, mode = "Additive", geno, covariate, kz = 20, kb = 10, smooth.cov = FALSE)
MetaMFLM_fpca_no_position(L, is.homo = TRUE, pheno, mode = "Additive", geno, covariate, kz = 20, kb = 10, smooth.cov = FALSE)

MetaMFLM_fpca_no_position(L, is.homo = FALSE, pheno, mode = "Additive", geno, covariate, kz = 20, kb = 10, smooth.cov = FALSE)
MetaMFLM_fpca_no_position(L, is.homo = FALSE, pheno, mode = "Additive", geno, covariate, kz = 20, kb = 10, smooth.cov = FALSE)

###
MetaMFLM_fpca(L, is.homo = TRUE, pheno, mode = "Additive", geno, covariate, pos, kz = 20, kb = 10, smooth.cov = FALSE)
MetaMFLM_fpca(L, is.homo = TRUE, pheno, mode = "Additive", geno, covariate, pos, kz = 20, kb = 10, smooth.cov = FALSE)

MetaMFLM_fpca(L, is.homo = FALSE, pheno, mode = "Additive", geno, covariate, pos, kz = 20, kb = 10, smooth.cov = FALSE)
MetaMFLM_fpca(L, is.homo = FALSE, pheno, mode = "Additive", geno, covariate, pos, kz = 20, kb = 10, smooth.cov = FALSE)

###
MetaMFLM_add_effect(L, is.homo = FALSE, pheno, mode = "Additive", geno, covariate)

### The following function is not working since geno[[k]] have different number of columns ###

MetaMFLM_add_effect(L, is.homo = TRUE, pheno, mode = "Additive", geno, covariate)

